//
//  galefunctions.h
//  
//  Authors:  Daniel J. Bates, Jonathan D. Hauenstein, Matthew E. Niemerg and Frank Sottile
//
//  Code Last modified: 8/25/2013
#ifndef GALEFUNCTIONS_H_
#define GALEFUNCTIONS_H_



#ifndef VERBOSE
#define VERBOSE 0
#endif

#include <math.h>
#include <cmath>
#include "signpattern.h"
#include "functions.h"
#include "mtrand.hpp"
#include <iostream>
#include <stdlib.h>

void writeAlphaCertifiedSettings(int algorithm, int arithmetictype, int precision, int refinedigits, 
				 int numrandomsystems, int randomdigits, int newtononly, int numiterations,int realitycheck, int realitytest){

  //  std::cout << "Entering writeAlphaCertifiedSettings(a bunch of stuff)\n\n";

  std::ofstream fout("alphaCertifiedSettings");

  fout << "ALGORITHM: " << algorithm << ";\n"
       << "ARITHMETICTYPE: " << arithmetictype << ";\n"
       << "PRECISION: " << precision << ";\n"
       << "REFINEDIGITS: " << refinedigits << ";\n"
       << "NUMRANDOMSYSTEMS: " << numrandomsystems << ";\n"
       << "RANDOMDIGITS: " << randomdigits << ";\n"
       << "NEWTONONLY: " << newtononly << ";\n"
       << "NUMITERATIONS: " << numiterations << ";\n"
       << "REALITYCHECK: " << realitycheck << ";\n"
       << "REALITYTEST: " << realitytest << ";\n";
  fout.close();
  //std::cout << "Leaving writeAlphaCertifiedSettings(a bunch of stuff)\n\n";
}


// this function is step3 of algorithm2
// here exponents is (numlinear - numparam) x numlinear
// could possibly do this for all solutions so we don't have
// to redo the GE for each solution!
// not implemented yet, would just have to do a std::vector< std::vector<double> > AllSolutions

std::vector<std::vector<std::pair<double, double> > > SolveLinears(int** Exponents,
								   std::vector< std::vector<std::pair<double, double> > > AllSolutions,
								   int numlinear, int numparam){


  std::cout << "numlinear = " << numlinear << ", numparam = " << numparam << "\n\n";


  std::vector<std::vector<std::pair<double, double> > > NonExponentiatedSolutions;                                                                   
  // double check AllSolutions.size() = numlinear -- if not, exit early and double check                                                    
  // as this should never happen               
  //  std::cout << "AllSolutions.size() = " << AllSolutions.size() << "\n";
  // std::cout << "AllSolutions[0].size() = " << AllSolutions[0].size() << "\n";
  for (int i = 0; i < AllSolutions.size(); ++i){                                                                                          
    if (AllSolutions[i].size() != numlinear){                                                                     
      //    std::cout << "Houston, we have a problem. In SolveLinears - step3 of algorithm2.\n";
      // std::cout << "AllSolutions[" << i << "] = " << AllSolutions[i].size() << " != " << numlinear-numparam << "\n";
      return NonExponentiatedSolutions; // returns an empty vector of doubles ...  
    }                                                                                                     
  }  
  /*
  for (int i = 0; i < AllSolutions.size(); ++i){
    std::cout << "x_" << i+1 << "* \n";
    for (int j = 0; j < AllSolutions[i].size(); ++j){
      std::cout << "L" << j+1 << " = " << AllSolutions[i][j].first
                << " + " <<  AllSolutions[i][j].second << "*I" << "\n";
    }
  }
  */


  // now create a double** that will apply the gaussian elimination algorithm

  // numrows = numlinear-numparam
  // numcolumns = numlinear + AllSolutions.size()*2 (*2 for the imaginary component)
#ifdef debugmem
    std::cout << "\n******\n\nallocating ToGaussianize\n\n******\n";
#endif
  double** ToGaussianize =  new double*[numlinear]; // new double*[numlinear-numparm]; // so rows is numlinear-numparam
  
  for (int i = 0; i < numlinear; ++i){ // numlinear-numparam; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating ToGaussianize[" << i << "]\n\n******\n";
#endif
    ToGaussianize[i] = new double[numlinear + AllSolutions.size()*2];
    
    for (int j = 0; j < numlinear; ++j){
      // std::cout << "i = " << i << ", j = " << j << "\n\n";
      ToGaussianize[i][j] = double(Exponents[i][j]);

    }
    for (int j = 0; j < AllSolutions.size()*2; ++j){
      //     std::cout << "i = " << i << ", j = " << j + numlinear << "\n\n";
      ToGaussianize[i][j + numlinear] = double(AllSolutions[(j/2)][i].first);
      ToGaussianize[i][j + 1 + numlinear] = double(AllSolutions[(j/2)][i].second);
      ++j;

    }
  }


  // output before the logarithmizing
  //  std::cout << "The augmented exponent matrix before the logarithmizing.\n\n";
  
  

    //
  //  logarithmizing the values
  for (int i = 0; i < numlinear; ++i){ //numlinear - numparam; ++i){
    // for (int j = 0; j < numlinear; ++j){
    //std::cout << ToGaussianize[i][j] << " "; 
    //}
      
      
    for (int j = 0; j < AllSolutions.size()*2; ++j){
      double curreal = ToGaussianize[i][j + numlinear];
      double curimag = ToGaussianize[i][j + 1 + numlinear];

      double theabs = pow(abs(pow(curreal,2.0) - pow(curimag,2.0)), .5);
      // the angle theta
      double theta = atan2(curimag,curreal);
      ToGaussianize[i][j + numlinear] = log(theabs);
      ToGaussianize[i][j + 1 + numlinear] = theta;
      ++j;
      // std::cout << curreal << " " << curimag << (j != AllSolutions.size()*2 - 1 ? " " : "\n");
      //      if (curreal < 0){
      //	std::cout << "curimag = " << curimag << ", curreal = " << curreal << ", atan2(curimag,curreal) = " << atan2(curimag,curreal) << "\n";
  
 
    //}
    } // end for j
      
      
      
      
      
  }  // end for i

 
 
  //  std::cout << "Exponent matrix augmented before the transpose of the numlinear - numparam rows / cols.\n\n";                     
  //for (int i = 0; i < numlinear; ++i){ //numlinear-numparam; ++i){                                                                            
  // for (int j = 0; j < numlinear+AllSolutions.size()*2; ++j){                                                       
  //   std::cout << ToGaussianize[i][j] << ( j != numlinear+AllSolutions.size()*2 - 1 ? " " :"\n" );  
  // }                                                                                                                            
  // }  


  // now since the solutions augmented on the matrix correspond to the linears with the values substituted back in
  // i.e. not to the values of the solved parameters
  // since the solved parameters exponents go from numlinear - numparam + 1 ... numlinear columns
  // we can just transpose the portion of the matrix necessary

    
 for (int i = 0; i < numlinear; ++i){ // numlinear - numparam; ++i){
    for (int j = i; j < numlinear; ++j){// numlinear - numparam; ++j){
      double tmp = ToGaussianize[i][j];
      ToGaussianize[i][j] = ToGaussianize[j][i];
      ToGaussianize[j][i] = tmp;
    }
  }


 /*  std::cout << "Exponent matrix after the transpose and before the gaussian elimination.\n\n";
  for (int i = 0; i < numlinear; ++i){ //numlinear - numparam; ++i){
    for (int j = 0; j < numlinear + AllSolutions.size()*2; ++j){
      std::cout << ToGaussianize[i][j] << ( j != numlinear+AllSolutions.size()*2 - 1 ? " " : "\n" );
    }
  }
 */

  //  GaussianEliminationT(ToGaussianize, numlinear-numparam, numlinear+AllSolutions.size()*2);
  GaussianEliminationT(ToGaussianize, numlinear, numlinear+AllSolutions.size()*2);

  // output the vector after all the gaussian elimination
  /*  
  std::cout << "Augmented form after the gaussian elimination.\n\n";
  for (int i = 0; i < numlinear; ++i){//numlinear-numparam; ++i){
    for (int j = numlinear; j < numlinear+AllSolutions.size()*2; ++j){
      std::cout << ToGaussianize[i][j] << ( j != numlinear+AllSolutions.size()*2 - 1 ? " " :"\n" );
    }  
  }
  */
    
    
    /*
    std::cout << "Augmented matrix before the logarithmizing.\n\n";
    
    for (int i = 0; i < numlinear; ++i){ //numlinear - numparam; ++i){
        for (int j = 0; j < numlinear; ++j){
            std::cout << ToGaussianize[i][j] << " ";
        }
        
        
        for (int j = 0; j < AllSolutions.size()*2; ++j){
            double curreal = ToGaussianize[i][j + numlinear];
            double curimag = ToGaussianize[i][j + 1 + numlinear];
            
            double theabs = pow(abs(pow(curreal,2.0) - pow(curimag,2.0)), .5);
            // the angle theta
            double theta = atan2(curimag,curreal);
            ToGaussianize[i][j + numlinear] = log(theabs);
            ToGaussianize[i][j + 1 + numlinear] = theta;
            ++j;
            std::cout << curreal << " " << curimag << (j != AllSolutions.size()*2 - 1 ? " " : "\n");
            //      if (curreal < 0){
            //	std::cout << "curimag = " << curimag << ", curreal = " << curreal << ", atan2(curimag,curreal) = " << atan2(curimag,curreal) << "\n";
            
            
            //}
        } // end for j
    }  // end for i
  

    
    
    
   
    
    std::cout << "Augmented form after the logarithmizing.\n\n";
    for (int i = 0; i < numlinear; ++i){//numlinear-numparam; ++i){
        for (int j = numlinear; j < numlinear+AllSolutions.size()*2; ++j){
            std::cout << ToGaussianize[i][j] << ( j != numlinear+AllSolutions.size()*2 - 1 ? " " :"\n" );
        }
    }
    
    */
    
    
    
    
    
    
    
  // this for loop puts things in the right order!
  for (int j = 0; j < AllSolutions.size()*2; ++j){
    std::vector<std::pair<double,double> > CurSolution;                                                                                              
    
    for (int i = 0; i < numlinear; ++i){//numlinear - numparam; ++i){                                                                                      
      std::pair<double,double> CurValue(ToGaussianize[i][numlinear + j], ToGaussianize[i][numlinear + j + 1]);
      CurSolution.push_back(CurValue);
      
      //if (j == AllSolutions.size()*2){
      //	delete[] ToGaussianize[i]; 
      //}
    }
    ++j;
    NonExponentiatedSolutions.push_back(CurSolution);                                                
    
  }  
  delete[] ToGaussianize; 
  return NonExponentiatedSolutions;
}


std::vector<std::pair<double, double> > ExponentiateVector(std::vector<std::pair<double, double> > solutions){             
  std::vector<std::pair<double, double> > ExponentiatedSolutions;                                         
  for (int i = 0; i < solutions.size(); ++i){

    //    std::cout << i << "\n";
    // std::cout << solutions[i].first*cos(solutions[i].second) << " + " << solutions[i].first*sin(solutions[i].second) << "*I\n";
    ExponentiatedSolutions.push_back(std::pair<double,double>(exp(solutions[i].first)*cos(solutions[i].second),     
                                                              exp(solutions[i].first) * sin(solutions[i].second) ) );
    // std::cout << "Exponentiation = " << exp(solutions[i].first)*cos(solutions[i].second) << " + " 
    //      << exp(solutions[i].first)*sin(solutions[i].second)
    //	      << "*I\n";
    // std::cout << solutions[i].second << "\n";
    // std::cout << "cos(theta) = " << cos(solutions[i].second) << ", sin(theta) = " << sin(solutions[i].second) << "\n";
  }                                                                                                                                
  return ExponentiatedSolutions; 
}   



int ParseInputGale(std::string filename, Variable* &All_Variable_Ptr, Rational** &LC, int** &exponents,
		   int & numparam, int & numlinear){

  std::ifstream file(filename.c_str());
  if (!file.is_open()){

    std::cout << "Invalid filename. Exitting program.";
    std::cout << std::endl;
    return -1;
  }


  // all this stuff needs to be changed for m > 0
  // this is not right
  
  // but when m = 0
  // numparam = # basis vectors 
  // and
  // numlinears = rows of LC
  // basislength =length of basis vectors in exponents
  // so file starts off as : 
  // numparam
  // numlinear
  // int numparam, numlinear;
  
  //  std::cout 
  std::string line;
  getline(file, line);
  std::stringstream ss3;
  ss3 << line;
  ss3 >> numparam;
  ss3.clear();
  ss3.str("");
  getline(file, line);
  ss3 << line;
  ss3 >> numlinear;
  //  file >> numparam >> numlinear;
  ss3.clear();
  ss3.str("");
  //  getline(file,line);
  //ss3 << line;
  // ss3 >> basislength
  // std::cout << "Inside ParseInputGale\n\n" << "numparam = " << numparam << "\nnumlinear = " << numlinear
  //    << "\n";
    
#ifdef debugmem
    std::cout << "\n******\n\nallocating exponents\n\n******\n";
#endif
  exponents = new int*[numparam];  // is this right? need to be able to deal
                              // with m \neq 0
  
  for (int i = 0; i < numparam; ++i){
    exponents[i] = new int[numlinear];
    std::string tmp;
    
    getline(file, tmp);
    // put the tmp string onto a stringstream
    std::stringstream ss;
    ss << tmp;
    // std::cout << "extracted string : = " << tmp << "\n";
    for (int j = 0; j < numlinear; ++j){
      ss >> exponents[i][j];
      
      // std::cout << "exponents[" << i << "][" << j << "] = " << exponents[i][j] << "\n";
    }

  }
  // now read in the next line
  // that holds the variables
  std::string tmp;
  getline(file,tmp);
  // place the string on a stream
  // so as to set the variable pointer

  std::stringstream ss;
  ss << tmp;
  //   std::cout << "line that holds the variables: " << tmp << "\n";
  // now place this all in an array that holds the variables
#ifdef debugmem
    std::cout << "\n******\n\nallocating All_Variable_Ptr\n\n******\n";
#endif
  All_Variable_Ptr = new Variable[numparam];
  
  for (int i = 0; i < numparam; ++i){
    std::string token;
    ss >> token;
    Variable CurrentVar(token);
    All_Variable_Ptr[i] = CurrentVar;
    // print out the variables (for now)
    //  std::cout << i << " = " << All_Variable_Ptr[i] << "\n";
  }
  
  // now for each line, place the them into the rational LC array
  //  std::cout << "numlinear = " << numlinear << " in ParseInputGale\n";
  //  LC = new Rational*[numlinear];
    
#ifdef debugmem
    std::cout << "\n******\n\nallocating LC\n\n******\n";
#endif
  LC = new Rational*[numparam + 1];
  // std::cout <<  "At the point of populating the LC array in ParseInputGale \n\n";
  //std::cout << "numlinear = " << numlinear << ", numparam = " << numparam << "\n";
  
  for (int i = 0; i < numparam + 1; ++i){ // was numlinear
#ifdef debugmem
      std::cout << "\n******\n\nallocating LC[" << i << "]\n\n******\n";
#endif
    LC[i] = new Rational[numlinear+1]; // was numparam + 1
    // get the line and place on a string
    std::string mytmp;
    getline(file,mytmp);
    std::stringstream ss2;
    ss2 << mytmp;
    //std::cout << "line read in for LC : " << mytmp << "\n";
    for (int j = numparam + 1; j <  numlinear + 1;++j){ // was j = 0; j < numparam+ 1
      // std::cout << "i = " << i << ", j = " << j << "\n\n";
      ss2 >> LC[i][j];
      //      std::cout << "LC[" << i << "][" << j << "] = " << LC[i][j] << "\n";
    }
    LC[i][i] = 1;
  }
  
  // print out the LC array to make sure it was loaded correctly 
  for (int i = 0; i < numparam + 1; ++i){
    
    for (int j = 0; j < numlinear + 1; ++j){
          std::cout << LC[i][j] << " "; 
    }
      std::cout << std::endl;
  }
  
  //  std::cout << "returning from ParseInputGale\n\n";
  return 1; // yay!
}


std::vector<std::vector<std::pair<double, double> > > ReadUnique(std::string uniquefilename, int numparam){

  // get the unique solutions -- i.e. don't double up if multiplicity > 1                                                                                                   
  std::ifstream fin(uniquefilename.c_str());

  int numsolutions;
  std::string line;
  getline(fin, line);
  std::stringstream ss;
  ss << line;
  ss >> numsolutions;
  ss.clear();
  ss.str("");
  std::vector<std::vector<std::pair<double, double> > > AllSolutions;
  for (int i = 0; i < numsolutions; ++i){
    getline(fin, line); // for the space                                
    std::vector<std::pair<double,double> > curvalue;

    for (int i = 0; i < numparam; ++i){
      getline(fin,line);
      double rvalue;
      double ivalue;
      ss << line;
      ss >> rvalue;
      ss >> ivalue;

      ss.clear();
      ss.str("");
      std::pair<double,double> curdim(rvalue,ivalue);
      curvalue.push_back(curdim);
    }
    AllSolutions.push_back(curvalue);
  }

  return AllSolutions;


}


std::pair<double,double>  EvaluateLinear(std::vector<std::pair<double, double> > solution,
					 Rational* LC){
 
  double rvalue = 0;
  double ivalue = 0;
  for (int i = 0; i < solution.size(); ++i){
    double LCVal = double(LC[i].GetNumerator()) / double(LC[i].GetDenominator());
    rvalue+=LCVal*solution[i].first;
    ivalue+=LCVal*solution[i].second;
  }
  double LCVal = 
    double(LC[solution.size()].GetNumerator()) / double(LC[solution.size()].GetDenominator());
  rvalue+=LCVal;
  std::pair<double, double> Done(rvalue,ivalue);

  return Done;
}



std::vector<std::pair<double, double> > EvaluateLinears(std::vector<std::pair<double,double > > solution,
							Rational** LC,  int numlinear){
  
  std::vector<std::pair<double,double> > CurrentLinears;
  for (int i = 0; i < numlinear; ++i){
    std::pair<double,double> CurLinear = EvaluateLinear(solution, LC[i]);
    CurrentLinears.push_back(CurLinear);
  }
  return CurrentLinears;

}

std::vector<std::vector<std::pair<double, double> > > EvaluateAll(std::vector<std::vector<std::pair<double, double > > > AllSolutions,
								  Rational** LC, int numlinear){

  std::vector<std::vector<std::pair<double,double> > > AllLinears;
  for (int i = 0; i < AllSolutions.size(); ++i){
    std::vector<std::pair<double, double > > CurrentLinears = EvaluateLinears(AllSolutions[i], LC, numlinear);
    AllLinears.push_back(CurrentLinears);
  }
  return AllLinears;
}

std::vector<std::vector<std::pair<double, double> > > CompleteUnwrap(int** Exponents,
                                                                     Rational** LC,
                                                                     std::vector<std::vector<std::pair<double,double> > > AllGTSolutions,
                                                                     int numlinear, int numparam){
  // this takes AllGTSolutions and evaluates the linears used in the gale dual system                                                                                                       
  // So AllLinearsEvaluated.size() = AllGTSolutions.size() and AllLinearsEvaluated[i].size() = numlinear - numparam (?) -- i.e. only need evaluation of parametrization                     

  std::vector<std::vector<std::pair<double, double> > > AllLinearsEvaluated = EvaluateAll(AllGTSolutions, LC, numlinear);
  

  // this next function goes ahead and augments the exponents matrix                                                                                                                        
  // with ALL the solutions of the linears tacked on                                                                                                                                        
  // then ONE solve is necessary to get to the non-exponentiated form of the solutions                                                                                                      
  // i.e. essentially we have Ax = b, but multiple b's.                                                                                                                                     
  // augment A with all the b's (in this case, complex, but note that since A is the exponent matrix, no complex part there so yay!                                                         
  // Then perform row operations on A to make it into the identity (or near the identity since the numrows < numcolumns, but this is good if we have                                        
  // some non-LI columns)                                                                                                                                                                   
  // then all of the evaluation of the linears are now solved for Ax = b, i.e. we now have for each b_i x_i = A^{-1}b_i (implicitly)                                                     


  std::vector<std::vector<std::pair<double,double> > > XSolutions = SolveLinears(Exponents,
                                                                                 AllLinearsEvaluated,
                                                                                 numlinear, // technically this is total number of linears, but we ignore the last numparam              
                                                                                 numparam); // as that is just evaluation at the solution ... actually previous function ignores it?    
  // now just apply the exponentiation on the XSolutions                                                                                                                                    
  // this algorithm takes into account if there is an imaginary                                                                                                                             
  // component, i.e.                                                                                                                                                                        
  // exp(p_r)*exp(p_i) = exp(p_r)+cos(p_i) + i*sin(p_i)                                                                                                                  
  // which is returned as a std::pair<double,double>                                                                                                                      
  std::vector<std::vector<std::pair<double, double> > > ExponentiatedSolutions;
  for (int i = 0; i < XSolutions.size(); ++i){

    std::vector<std::pair<double,double> > CurExponentiatedS = ExponentiateVector(XSolutions[i]);
    ExponentiatedSolutions.push_back(CurExponentiatedS);
  }
  return ExponentiatedSolutions;
}





// note this string is in bertini output for functions
// vector holds the functions, 1 in each slot

// for the exponents is a numparam x basislength = numparam x numlinear
// LC = numlinear x numparam + 1
std::vector<std::string> GaleSystemToString(Variable* All_Variable_Ptr, Rational** LC, int** exponents, int numparam, int numlinear){

  
  std::vector<std::string> TheFuncts;
  
  for (int j = 0; j < numparam; ++j){
    std::string pos_str = "";
    std::string neg_str = "";
    for (int i = 0; i < numlinear; ++i){
      std::string curstring = "(";
      for (int k = 0; k < numparam; ++k){
	if (LC[i][k] != 0){
	  if (k != 0 && curstring != "("){	   
	    curstring.append(" + ");
	  }
	  std::stringstream LCss;
	  LCss << (i < numlinear - numparam ? -1*LC[i][k]: LC[i][k]) << "*" << All_Variable_Ptr[k];
	  curstring.append(LCss.str());
	} // end if LC[i][k] = 0
      } // end k < numparam
      if (LC[i][numparam] != 0){
	if (curstring != ""){
	  curstring.append(" + ");
	  std::stringstream LCss2;
	  LCss2 << -1*LC[i][numparam];
	  curstring.append(LCss2.str());
	}
      }
      curstring.append(")^(");
      if (exponents[j][i] < 0){
	std::stringstream LCe;
	LCe  << -1*exponents[j][i];
	curstring.append(LCe.str());
	curstring.append(")");
	if (neg_str != ""){
	  neg_str.append("*");
	}
	neg_str.append(curstring);
      }
      else if (exponents[j][i] > 0){
	std::stringstream LCe2;
	LCe2 << exponents[j][i];
	curstring.append(LCe2.str());
	curstring.append(")");
	if (pos_str != ""){
	  pos_str.append("*");
	}
	pos_str.append(curstring);
      }
      else { // exponent is 0, do nothing
      } // end testing the positivity of exponents 
    } // end running through length of exponents
    std::string master_f = "";
    master_f.append(pos_str);
    if (neg_str != ""){
      master_f.append(" - ");
      master_f.append(neg_str);
    }
    TheFuncts.push_back(master_f);
  }

  return TheFuncts;

}

// for the exponents is a numparam x numlinear                                                                                                                    
// LC = numlinear x numparam + 1 

void WriteGaleSystemToBertini(std::string filename, 
			      Variable* All_Variable_Ptr, 
			      Rational** LC, 
			      int** exponents, 
			      int numparam, int numlinear){



  std::vector<std::string> MyFuncts = GaleSystemToString(All_Variable_Ptr, LC,
							 exponents, numparam,
							 numlinear);

  // Variable contains the names of the parameters ... could just ignore that
  // and call them s0 ... s(numparam-1) ?
  // really it doesn't matter what these are to bertini

  std::string inputfname = "input_";
  inputfname.append(filename);
  std::ofstream fout(inputfname.c_str());
  fout << "CONFIG\n";
  // default values for now

  fout << "END;\n";
  
  fout << "variable_group ";
  for (int i = 0; i < numparam; ++i){
    fout << All_Variable_Ptr[i] <<  (i != numparam -1 ? ", " : ";\n");
  }
  fout << "function ";
  for (int i = 0; i < MyFuncts.size(); ++i){
    fout << "f" << i+1 << (i != MyFuncts.size() - 1 ? ", " : ";\n");
  }
  for (int i = 0; i < MyFuncts.size(); ++i){
    fout << "f" << i+1 << " = " << MyFuncts[i] << ";\n";
  }
  fout << "END;";
  fout.close();
}


int ParseInputOriginal(std::string filename, int & numeq, int & numvar, int & nummon, Monomial* All_Mon_Ptr, 
               Variable* &All_Variable_Ptr, Polynomial* &All_Polynomials, Rational** &LC, int** &exponents){
    
    std::ifstream file(filename.c_str());
    if (!file.is_open()){
        
        std::cout << "Invalid filename. Exitting program.";
        std::cout << std::endl;
        return -1;
    }
    
    int l, m,n;
    
    file >> l >> m >> n;
    numeq = n;
    numvar = m+n;
    nummon = l+m+n;

    // add one for the constant
    // store trhe monomials and the variables in a pointer array of the appropriate sizes
#ifdef debugmem
    std::cout << "\n******\n\nallocating All_Mon_Ptr\n\n******\n";
#endif
    All_Mon_Ptr = new Monomial[nummon+1];
#ifdef debugmem
    std::cout << "\n******\n\nallocating All_Variable_Ptr\n\n******\n";
#endif
    All_Variable_Ptr = new Variable[numvar];
    // dummy string
    std::string current;
    getline(file, current);
    getline(file,current);
    
#ifdef VERBOSE
    /*    std::cout << std::endl;
    std::cout << current  << std::endl;
    std::cout << std::endl;
    */
#endif
    stringstream ss;
    ss << current;
    std::string current_mon_str;

    
    // while we parse the monomial string, we go ahead and find all the variables during the same time
    // as parsing the monomials, index is for setting the specific monomial and varsfound is
    // a check on placing the variables in the right order and not rechecking each time if we're already
    // done
    int index = 0;
    int varsfound = 0;

    
    
    while (ss >> current_mon_str){
        // the monomial strings are space delimited
        // send them to the monomial's function set to set
        // the current index
      
      All_Mon_Ptr[index].Set(current_mon_str);
        
        
        if (varsfound < numvar){
            
            // find the variable if it's located in the array
            
            for (int i = 0; i < All_Mon_Ptr[index].size; ++i){
                
                Variable current_var = All_Mon_Ptr[index].Variables[i];
                int loc = binarySearch(All_Variable_Ptr, 0, 
                                       varsfound, 
                                       current_var);
                if (loc < 0){
                    
                    // place to insert is loc = -insert-1, so
                    // insert = -loc - 1
                    int insert = -1*loc - 1;
                    // if the insert location is not at the end
                    // bubble the other ones down and place in the right location
                    
                    if (insert < varsfound){
                        
                        for (int j = varsfound; j >= insert+1; --j){
                            All_Variable_Ptr[j] = All_Variable_Ptr[j-1];
                        }
                        
                    }
                    
    
                    
                    
                    // set the current variable found to the right position
                    All_Variable_Ptr[insert] = current_var;
                    ++varsfound;	// increase the number of variables found
                }	
            }
        }
        ++index;  // increase the index for the monomials
    }

    
    
    
#if VERBOSE // output stuff    
    std::cout << "The variables in order are : " << std::endl << std::endl;
    for (int i = 0; i < varsfound; ++i){
        std::cout << i << " : ";
        std::cout << All_Variable_Ptr[i] << std::endl;
    }
    std::cout << std::endl;
    std::cout << "The monomials are : " << std::endl << std::endl;
    for (int i = 0; i < nummon; ++i){
        std::cout << All_Mon_Ptr[i] << std::endl;
    }
    std::cout << std::endl;
#endif
    
    // now read in LC array
    
    std::string LCline = ""; // dummy string for parsing
#ifdef debugmem
    std::cout << "\n******\n\nallocating LC\n\n******\n";
#endif
    LC = new Rational*[numeq]; // The rational coefficient array
    // current index for placing the polynomials in All_Polynomials 
    // pointer array
    int cindex = 0;
#ifdef debugmem
    std::cout << "\n******\n\nallocating All_Polynomials\n\n******\n";
#endif
    All_Polynomials = new Polynomial[numeq];
    

    
    // read in the rationals a line at a time, delimited by a space
    while (getline(file,LCline)){
#ifdef debugmem
        std::cout << "\n******\n\nallocating LC[" << cindex << "]\n\n******\n";
#endif
        LC[cindex] = new Rational[nummon+1];
        // place the LC string onto a stringstream
        stringstream s;
        s << LCline;
        for (int j = 0; j < nummon+1; ++j){
            Rational cur_rat = Rational();
            // use the extraction operator on the current 
            //rational, placing
            s >> cur_rat;  
            // the value that's on the string to the value in the LC array
            LC[cindex][j] = cur_rat;
        }
        // set the polynomial P to the correct value with the constructor
        Polynomial P(All_Mon_Ptr, LC[cindex], nummon+1);
        // set the value of the polynomial in the array to the right value
        All_Polynomials[cindex] = P;          
        ++cindex;    
    }
    
    
    // create the exponents array
#ifdef debugmem
    std::cout << "\n******\n\nallocating exponents\n\n******\n";
#endif
    exponents = new int*[numvar];
    for (int k = 0; k < numvar; ++k){
#ifdef debugmem
        std::cout << "\n******\n\nallocating exponents[" << k << "]\n\n******\n";
#endif
        exponents[k] = new int[nummon];
        for (int j = 0; j < nummon; ++j){
            exponents[k][j] = 0;
        }
    }
    
    for (int j = 0; j < nummon; ++j){
        for (int k = 0; k < All_Mon_Ptr[j].size; ++k){
            
            Variable current_variable;
            current_variable = All_Mon_Ptr[j].Variables[k];
            int var_loc = 
            binarySearch(All_Variable_Ptr, 0,numvar - 1, current_variable);
            exponents[var_loc][j] = All_Mon_Ptr[j].Exponents[k];
        }
    }
#if VERBOSE
    std::cout << "The exponents are : " << std::endl;
    for (int j = 0; j < numvar; ++j){
        printArray(exponents[j], nummon);
        std::cout << std::endl;
    }
    std::cout << "The LC array is : \n\n";
    for (int j = 0; j < numeq;++j){
        
        printArray(LC[j],nummon+1);
        std::cout << "\n";
    }
#endif
    

    return 1; // 1 is success!
}


void WriteGAPoutputOriginal(int numeq, int numvar, int nummon, int** exponents){
        
    ofstream file2;
    file2.open("./exp.dat");
    file2 << "A := [[";
    for (int i = 0; i < numvar; ++i){
        for (int j = 0; j < nummon; ++j){
            file2 << exponents[i][j]
            << (j == nummon - 1 ? "" : ", ");
        }
        file2 << " ]" << (i == numvar - 1 ? "": ", [");
        
    }
    file2 << "];\n";
    
    
    // find the null space
    // for (int i = 0; i < numeq; ++i){
    file2 << "n := NullspaceIntMat(TransposedMat(A));\n";
    // now take the nullspace guys
    // and perform LLL on them
    file2 << "reduced" << " := LLLReducedBasis(n).basis;\n";
    // }
    file2 << "LoadPackage(\"IO\");\n";
    file2 << "f := IO_File(\"shortnull.dat\", \"w\");\n";
    file2 << "s := \"\";";
    file2 << "for i in [1..Length(reduced)] do\n"
	<< "\tfor j in [1..Length(reduced[i])] do\n"
	<< "\t\tAppend(s, String(reduced[i][j]));\n"
	<< "\t\tAppend(s, \" \");\n"
	<< "\tod;\n"
	<< "Append(s, \"\\n\");\n"
	<< "od;\n";
    file2 << "IO_Write(f, s);\n";
    file2 << "IO_Flush(f);\n";
    file2 << "IO_Close(f);\n";
    file2 << "QUIT;";
    
    file2.close();
    
}

void WriteGAPoutputGale(int** exponents, int numparam, int numlinear){
  std::ofstream file("exp.dat");

  file << "A := [[";
  for (int i = 0; i < numparam; ++i){
    for (int j = 0; j < numlinear; ++j){
      file << exponents[i][j] << (j != numlinear - 1 ? ", " : "]");
    }
    file << (i != numparam - 1 ? ",[" : "];\n");
  }

  file << "reduced" << " := LLLReducedBasis(A).basis;\n";
  file << "LoadPackage(\"IO\");\n";
  file << "f := IO_File(\"shortnull.dat\", \"w\");\n";
  file << "s := \"\";";
  file << "for i in [1..Length(reduced)] do\n"
       << "\tfor j in [1..Length(reduced[i])] do\n"
       << "\t\tAppend(s, String(reduced[i][j]));\n"
       << "\t\tAppend(s, \" \");\n"
       << "\tod;\n"
       << "Append(s, \"\\n\");\n"
       << "od;\n";
  file << "IO_Write(f, s);\n";
  file << "IO_Flush(f);\n";
  file << "IO_Close(f);\n";
  // now here, compute the NullSpace and use it
  file << "n := NullspaceIntMat(TransposedMat(reduced));\n";
  file << "s2 :=\"\";"
       << "f2 := IO_File(\"supportmat.dat\", \"w\");\n"
       << "for i in [1..Length(n)] do \n"
       << "\tfor j in [1..Length(n[i])] do\n"
       << "\t\tAppend(s2, String(n[i][j]));\n"
       << "\t\tAppend(s2, \" \");\n"
       << "\tod;\n"
       << "Append(s2, \"\\n\");\n"
       << "od;\n"
       << "IO_Write(f2, s2);\n"
       << "IO_Flush(f2);\n"
       << "IO_Close(f2);\n";
  file << "QUIT;";
  

  
  file.close();
  
}

// the function that converts a gale system to a fewnomial system
// assuming input given is correct, i.c. all parametrizations and 
// choices are finished
// returns as the support matrix
int** ConvertGaleToFew(int** basis, int rows, int columns){

  
  // we need the null basis -- convert this via GAP to get the Monomials
  
  std::ofstream file("gap-input.dat");
  file << "A := [[";
  for (int i = 0; i < rows; ++i){
    for (int j = 0; j < columns; ++j){
      file << basis[i][j] << (j != columns - 1 ? ", " : "]");
    }
    file << (i != rows - 1 ? ", [" : "]\n");
  }
  
  file << "n := NullspaceIntMat(TransposedMat(A));\n";
  file << "reduced" << " := LLLReducedBasis(n).basis;\n";
  file << "LoadPackage(\"IO\");\n";
  file << "f := IO_File(\"support-matrix-temp.dat\", \"w\");\n";
  file << "s := \"\";";
  file << "Append(s, Length(reduced));\nAppend(s, \"\\n\");\nAppend(s, Length(reduced[1]);\n" 
       << "Append(s, \"\\n\");\n"
       << "for i in [1..Length(reduced)] do\n"
       << "\tfor j in [1..Length(reduced[i])] do\n"
       << "\t\tAppend(s, String(reduced[i][j]));\n"
       << "\t\tAppend(s, \" \");\n"
       << "\tod;\n"
       << "Append(s, \"\\n\");\n"
       << "od;\n";
  file << "IO_Write(f, s);\n";
  file << "IO_Flush(f);\n";
  file << "IO_Close(f);\n";
  file << "QUIT;";
  file.close();
  // now the support matrix is stored in a temp file with a column
  // indicates what the monomial is
  std::ifstream file2;
  file2.open("support-matrix-temp.dat");
  // file.open("support-matrix-temp.dat");
  
  std::string blank;
  // we know that this should be of a certain size
  // but, we didn't pass that information to this function
  // and we have to read that file anyway
  // and since it's only 2 line reads,
  // the info was saved in the output file
  int rows2;
  int columns2;
  std::string mystring;
  std::stringstream ss_to_conv;
  getline(file2,mystring);
  ss_to_conv << mystring;
  getline(file2, mystring);
  ss_to_conv << mystring;
  ss_to_conv >> rows2;
  ss_to_conv >> columns2;
#ifdef debugmem
    std::cout << "\n******\n\nallocating support_matrix\n\n******\n";
#endif
  int** support_matrix = new int*[rows2];
  for (int i = 0; i < rows; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating support_matrix[" << i << "]\n\n******\n";
#endif
    support_matrix[i] = new int[columns2];
  }
  int count = 0;
  while (getline(file2, blank)){
    std::stringstream ss;
    ss << blank;
    int curval;
    for (int i = 0; i < columns2; ++i){
      ss >> curval;
      support_matrix[count][i] = curval;
    }
    ++count;
  }

  // now the monomials are stored in the support_matrix
  // and there was much rejoicing in the land . . .
  // (and they ate robin's minstrels -- yay!)

  
  

  return support_matrix;
  // am i allowed to return a dynamically allocated object in this manner ?
  // good, i think
}

// solve a given linear system with real coefficients

Rational* Solution(Rational** LC, int l){

  // now run gaussian elimination on the LC rational array
  
  // create a copy of the array, so as not to disturb
  // the original data
#ifdef debugmem
    std::cout << "\n******\n\nallocating LC2\n\n******\n";
#endif

Rational** LC2 = new Rational*[l];
  for (int i = 0; i < l; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating LC2[" << i << "]\n\n******\n";
#endif
    LC2[i] = new Rational[l+1];
    for (int j = 0; j < l+1; ++j){
      LC2[i][j] = LC[i][j];      
    }
  }

  GaussianEliminationT(LC2, l, l + 1);
#ifdef debugmem
    std::cout << "\n******\n\nallocating ToReturn\n\n******\n";
#endif
  Rational* ToReturn = new Rational[l+1];
  for (int i = 0; i < l; ++i){
    
    ToReturn[i] = LC2[i][l]; // i.e. make the column a row (shouldn't this be -1*LC2[i][l] ? )
    delete[] LC2[i];
  }

  delete[] LC2; // do i need to delete each
                // row, too? perhaps ...

  return ToReturn;
}


// at this point there is no swapping of rows or columns for testing of
// different monomial parametrizations
// this will be done separately, i.e. pass the function what it ought to be passed
// and beforehand (or in the program) adjust LC stuff accordingly for
// different parametrizations


bool IsPositiveVertex(Vertex V, Rational** LC, int rows, int columns){

  // determine if a vertex is on the convex hull of the positive chamber
  // this is done by testing the evaluation of each of the linears that do not
  // form the vertex.
  // if all are positive, then it is a positive vertex
  // if one is negative, it is not
  // don't test linears used to determine the vertex
  // as these are 0, and that's extra computations we can avoid

  std::cout << "Entering IsPositiveVertex(V, LC, rows, columns)\n";

  std::cout << "The linears under the current parametrization are : \n\n";
  for (int i = 0; i < rows; ++i){
    for (int j = 0; j < columns; ++j){
      std::cout << LC[i][j] << "\t";
    }
    std::cout << "\n";
  }
  

  int supportinglinearcount = 0;
  int supportinglinear = V.second[supportinglinearcount];

  for (int i = 0; i < rows; ++i){

    Rational Value(0);
    if (i != supportinglinear){
      for (int j = 0; j < V.first.size(); ++j){
	Value += LC[i][j]*V.first[j];
      } // end for
      Value += LC[i][columns-1]; // add the constant
    } // end if
    else{
      // update current supportinglinear unless we are past the count
      // which is the same as columns - 1
      if (supportinglinearcount < columns){
	++supportinglinearcount;
	supportinglinear = V.second[supportinglinearcount];
      }
      else{
	supportinglinear = rows; // just update it to be = rows, so as not to worry about it anymore
      }
      // don't do the evaluation since we are on a supporting linear
    } 
    if (Value < Rational(0)){ // test that the Value is less than the Rational(0) -- force the rational < overloaded operator to be used
      std::cout << "Value = " << Value << " on linear " << i << " -- vertex does not create the positive orthinate\n\n";
      return false;
    }
  }
  return true;
}

std::vector<int> Click(std::vector<int> CurrentClick, std::vector<int> MaxClick){


  int currentlevel = CurrentClick.size() - 1;
  std::vector<int> Next = std::vector<int>(currentlevel + 1);
  for (int i = 0; i < currentlevel + 1; ++i){
    Next[i] = CurrentClick[i];
  }
  bool done = false;
  while (currentlevel >= 0 && !done){
    if (CurrentClick[currentlevel]+1 > MaxClick[currentlevel]){
      --currentlevel;
      for (int i = currentlevel+1; i < CurrentClick.size(); ++i){
	Next[i] = 0;
      } // end for
    } // end if
    else{
      Next[currentlevel] = CurrentClick[currentlevel]+1;
      done = true;
    } // end else if
 
    // test if the values stored in the higher indices are smaller than previous indices
    // if so, update accordingly
    int curmax = Next[0];
    for (int i = 1; i < Next.size();++i){
      if (Next[i] <= curmax){
	Next[i] = ++curmax;
      }
      else{
	curmax = Next[i];
      }
      
    } 
  } // end while
  return Next;

}

std::vector<Vertex> FindAllPositiveVertices(Rational** Linears, int l, int n){ // do i need to pass n?

  std::cout << "Inside FindAllPositiveVertices(Linears, l, n)\n";
  // now, we need to take all l-ple combinations of the linears
  // there are l + m + n (normally l + n, for m = 0) linears
  // i.e. there are a total of l + m + n choose l total
  // vertices, not all of which form the complement of the
  // hyperplane arrangement
  // note: l < n
  // and  (l + n)! / (l!n!) = (l + n) * (l + n - 1) * ... * (n + 1) / l! 
  // total vertices (this is a lot!!!!!) -- grows factorially ... boo
  // this is why this heuristic / preconditioning
  // is not practical
  // (although, we're only solving linears . . . so not nearly so bad)
  // use the Odometer Index function / semi-class
  // to get the set of linears to be used


  // essentially, the vertex will be computed from the lples
  // lples will store the n+l C l combinations of the lples.  
  // for i from 0 to (n+l C n) - 1
  //  lples[i] will contain l integers
  // these l integers correspond to which rows to use from
  // the Linears parameter
  // so as to determine that particular's combination vertex, associated with those linears

  // ****  NEED TO INCLUDE THE s0 = 0, s1 = 0, ..., s{l} = 0, linears!!!!!!!

  // use some of the multiplier functions found similarly in signpattern.h (included it as a header file)
  
  // set the MaxMultiplier to be n - l, n - l + 1, n - l + 2, ..., n - 1


  std::vector<int> MaxMultiplier(l);
  std::vector<int> Current(l);
  for (int i = 0; i < l; ++i){
    MaxMultiplier[i] = n + i;
    Current[i] = i;
  }

  /*  std::cout << "MaxMultiplier = " << "[";
  for (int i = 0; i < l; ++i){
    std::cout << MaxMultiplier[i] << (i != l - 1 ? ", " : "]\n");
  }
  */
  /******** OK ... so ... further down, if MaxMultiplier[i] > n (? is that right ...), then the linear
	    associated with that is s_{MaxMultiplier[i] - l} = 0 ****************/

  //  std::cout << "Here 1\n";
  std::vector<std::vector<int> > lples; // the l-ples of the linears
  bool equal = false; // this is for testing if Current == MaxMultiplier
  while (!equal){
    lples.push_back(Current);
    Current = Click(Current, MaxMultiplier);
    // test for equality of the MaxMultiplier (i.e. the max lple ordering possible)
    bool exitloop = false;
    for (int i = 0; i < Current.size() && !exitloop; ++i){
      if (Current[i] == MaxMultiplier[i]){
	equal = true;
      }
      else{
	equal = false;
	exitloop = true;
      }
    } // end for
  } // end while
  // push the MaxMultiplier on 
  lples.push_back(MaxMultiplier);
  // std::cout << "Here 2\n";
  // now we have all of the combinations of the linears
  // go ahead and do Gaussian Elimination on each combination
  // and test the vertices if positive or not
  // only store positive (obviously)
  
  std::vector<Vertex> PositiveVertices;
  
  
  std::cout << "Outputting all the " << l << "-ples:\n\n";
  
  for (int i = 0; i < lples.size(); ++i){
    std::cout << "lples[" << i << "] = [";
    for (int j = 0; j < lples[i].size(); ++j){
      std::cout << lples[i][j] << (j != lples[i].size() - 1 ? ", " : "]\n"); 
    }
    std::cout << "\n";
  }
#ifdef debugmem
    std::cout << "\n******\n\nallocating ToGaussianize\n\n******\n";
#endif
  Rational** ToGaussianize = new Rational*[n];
  
  for (int i = 0; i < n; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating ToGaussianize[" << i << "]\n\n******\n";
#endif
    ToGaussianize[i] = new Rational[n+l+1];
    for (int j = 0; j < n + l + 1; ++j){
      ToGaussianize[i][j] = Linears[i][j];
    } // end for j
  } // end for i

  //  std::cout << "l = " << l << ", n = " << n << "\n";
  std::cout << "Linears = \n\n";
 
  for (int i = 0; i < n; ++i){
    for (int j = 0; j < l+n+1; ++j){
      std::cout << Linears[i][j] << " ";
    }
    std::cout << "\n\n";
  }
  std::cout << "\n\n";
  

  //  std::cout << "ToGaussianize (Before Gaussianized) = \n\n";
  for (int i = 0; i < n; ++i){
    for(int j = 0; j <n + l + 1; ++j){
      //  std::cout<< ToGaussianize[i][j] << "\t";
    }
    //    std::cout << "\n\n";
  }


  // now we go ahead and Gaussianize the matrix, leaving Linears intact
  // std::cout << "n = " << n << ", l = " << l << "\n";
  GaussianEliminationT(ToGaussianize, n, n + l +1);

  // std::cout << "ToGaussianize (Gaussianized) = \n\n";
  for (int i = 0; i < n; ++i){
    for (int j = 0; j < n + l + 1; ++j){
      //  std::cout << ToGaussianize[i][j] << "\t";
    }
    //    std::cout << "\n\n";
  }


  //  std::cout << "Setting up Truncated Linears\n\n";

  // add on the linears that represent the coordinate axes
#ifdef debugmem
    std::cout << "\n******\n\nallocating TruncatedLinears\n\n******\n";
#endif
  Rational** TruncatedLinears = new Rational*[n+l]; // was only of size n
  for (int i = 0; i < n; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating TruncatedLinears[" << i << "]\n\n******\n";
#endif
    TruncatedLinears[i] = new Rational[l + 1];
    for (int j = 0; j < l + 1; ++j){
      TruncatedLinears[i][j] = -1*ToGaussianize[i][j+n]; // -1*(ToG[i][j+n]) -- since a monomial, say, c*d = linear, but in the LC matrix, everything is added together and = 0
    }
  }
  for (int i = 0; i < l; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating TruncatedLinears[" << i + n << "]\n\n******\n";
#endif
    TruncatedLinears[i+n] = new Rational[l + 1];
    for (int j = 0; j < l + 1; ++j){
      std::cout << "i = " << i << ", j = " << j << "\n";
      if (i == j){
	TruncatedLinears[i+n][j] = 1;
      }
    }
  }
  
  
  //  std::cout << "TruncatedLinears = \n\n";
  
  for (int i = 0; i < n+l; ++i){ // was just i < n
    for (int j = 0; j < l + 1; ++j){
      // std::cout << TruncatedLinears[i][j] << "\t";
    }
    //    std::cout << "\n\n";
  }
#ifdef debugmem
    std::cout << "\n******\n\nallocating ToGaussianize2\n\n******\n";
#endif
  Rational** ToGaussianize2 = new Rational*[l];
  for (int i = 0; i < l; ++i){
#ifdef debugmem
      std::cout << "\n******\n\nallocating ToGaussianize2[" << i << "]\n\n******\n";
#endif
    ToGaussianize2[i] = new Rational[l+1];
  }
  
  // now ToGaussianize2 will hold the truncated matrix
  // of ToGaussianize (the GaussianElimination of the Linears
  // and Linears is the current parametrization choice

  for (int i = 0; i < lples.size(); ++i){
    // reset ToGaussianize each time to the appropriate Linears
    // found in the LC array of the linears
    // lples[i] contains ALL the rows we want
    
    for (int j = 0; j < lples[i].size(); ++j){  // lples[i].size() == l
      // note that lples[i][j] will hold the current row
      // that we want to get from ToGaussianize
      //std::cout << "current row = " << lples[i][j] << "\n\n";
      for (int k = 0; k < l+1; ++k){
	// for (int ind = 0; ind < l + 1; ++ind){
	//	std::cout << "Taking from ToGaussianize (i.e. Linears Gaussianized), row " << lples[i][j]
	//	  << " and column " << k + n << "\n";

	/*
	if (lples[i][j] > l){
	  std::cout << "j = " << j << ", k = " << k << "\n";
	  ToGaussianize2[j][k] = TruncatedLinears[j][k];
	}
	else{
	  ToGaussianize2[j][k] = ToGaussianize[lples[i][j]][k+n]; // This is fine ... we are using the 
	  }*/

	ToGaussianize2[j][k] = TruncatedLinears[lples[i][j]][k];
	//	} // end for m
      } // end for k
    } // end for j

    // std::cout << "lples[" << i << "] = [";
    for (int j = 0; j < lples[i].size(); ++j){
      // std::cout << lples[i][j] << (j != lples[i].size() - 1 ? ", " :"]\n\n" );
    }

    //    std::cout << "ToGaussianize2 = \n\n";
    
    for (int j = 0; j < l; ++j){
      for (int k = 0; k < l+1; ++k){
	//	std::cout << ToGaussianize2[j][k] << "\t"; 
      }
      //      std::cout << "\n\n";
    }
      
    

    // now we perform GaussianElimination on ToGaussianize
    //   std::cout << "Here 3\n";
    GaussianEliminationT(ToGaussianize2, l, l+1);
    
    std::cout << "Here 4 (after GaussEl(ToGauss2))\n\n";

    std::cout << "ToGaussianize2 (after Gaussianize) = \n\n";
    
    for (int j = 0; j < l; ++j){
      for (int k = 0; k < l+1; ++k){
	std::cout << ToGaussianize2[j][k] << "\t";
      }
            std::cout << "\n\n";
    }



    // now take - of the last column and store it in a Vertex
    // this is just how this works . . . 
    Vertex V;
    std::vector<Rational> myV(l);
    //    Rational* myV = new Rational[l];
    for (int j = 0; j < l; ++j){
      myV[j] = -1*ToGaussianize2[j][l];
    }
    V.first = myV;
    std::vector<int> myL(l);
    for (int j = 0; j < l; ++j){
      myL[j] = lples[i][j];
    }

    V.second = myL;

    std::cout << "Here 4b after creation of Vertex V and before test of positive vertex.\n\n";
    std::cout << "The Vertex V is the point : (";
    for (int j = 0; j < V.first.size(); ++j){
      std::cout << V.first[j] << (j != V.first.size() - 1 ? ", " : ")\n\n");
    }
    std::cout << "With the Supporting Linears . . . \n\n";
    for (int j = 0; j < V.second.size(); ++j){
      std::cout << V.second[j] << (j != V.second.size() - 1 ? ", " : "\n\n");
    }

    if (IsPositiveVertex(V, TruncatedLinears, n+l, l+1)){
      std::cout << "V is a positive vertex.\n";
      PositiveVertices.push_back(V);
    }
    std::cout << "After Test Positive Vertex - iteration " << i << "\n\n";

  }
  

  //  std::cout << "here 5\n";

  delete[] ToGaussianize;
  delete[] ToGaussianize2;
  return PositiveVertices;
}



// now, we have a list of the vertices, and we keep track (should or will!)
// of which linears go with which vertex (so, at that point, who cares about order?)
// we really only want the vertices that form the positive
// chamber of the polytope (correct wording?)
// so in this case a vertex is a struct that is a Rational*, int*
// where int* is of size l, and Rational* is of size l
// yay, everything is nice and square (i.e. only m = 0, for now!)

std::vector<SignPattern> ComputeBestSigns(std::vector<SignPattern> OriginalBasis){

  /*
  std::ifstream fin;
  fin.open("shortnull.dat");
  
  std::string currentstr;
  
  std::vector<std::string> VecsAsStrings;
  
  while(getline(fin,currentstr)){ 
    VecsAsStrings.push_back(currentstr);         
  } 


  std::stringstream ss1;
  ss1 << VecsAsStrings[0];
  int dimensions = 0;
  int currentint1;
  while (ss1 >> currentint1){
    ++dimensions;
  }

  // create a vector for the original signpatterns                                                          

  std::vector<SignPattern> OriginalBasis = std::vector<SignPattern>(VecsAsStrings.size());
  

  // std::vector<int> CurrentIntVec = std::vector<int>(dimensions);                                         

  for (int i = 0; i < VecsAsStrings.size();++i){
    std::stringstream ss;
    ss << VecsAsStrings[i];
    int currentint;
    int count = 0;
    OriginalBasis[i].thesigns = std::vector<int>(dimensions);
    OriginalBasis[i].IntVec = std::vector<int>(dimensions);
    while (ss >> currentint){
      // CurrentIntVec[count] = currentint;                                                                 
      OriginalBasis[i].thesigns[count] = currentint % 2;
      OriginalBasis[i].IntVec[count] = currentint;
      ++count;
    }
  }

  */
  std::vector<int> MaxInEachDimension;
  int curmax = 0;

  for (int i = 0; i < OriginalBasis.size(); ++i){
    curmax = 0;
    for (int j = 0; j < OriginalBasis.size();++j){

      if (i !=j ){
        for (int k = 0; k < OriginalBasis[0].IntVec.size(); ++k){

          if (curmax < abs(OriginalBasis[j].IntVec[k])){
            curmax = abs(OriginalBasis[j].IntVec[k]);
          }
        }
      }
    }
    MaxInEachDimension.push_back(curmax);
  }


  std::vector<int> CurrentMultiplier;

  for (int i = 0; i < MaxInEachDimension.size(); ++i){
    CurrentMultiplier.push_back(-1*MaxInEachDimension[i]);
  }

  // set to original basis                                                                                  
  // otherwise, won't push those onto AllSigns                                                              
  std::vector<SignPattern> AllSigns = OriginalBasis;

  // push the -negative of the original basis onto AllSigns                                                 
  for (int i = 0; i < OriginalBasis.size();++i){
    int neg1 = -1;
    SignPattern CurNegative = neg1*OriginalBasis[i];
    AllSigns.push_back(CurNegative);
  }

  while (MaxElementV(CurrentMultiplier) != 0){
    SignPattern CSign = CurrentMultiplier[0]*OriginalBasis[0];
    for (int i = 1; i < CurrentMultiplier.size(); ++i){
      SignPattern AddSign = CurrentMultiplier[i]*OriginalBasis[i];
      CSign = CSign + AddSign;
    }

    // test if the multiplier was the same value in each position                                           
    int first = CurrentMultiplier[0];
    bool push = false;
 
      
    for (int i = 1; i < CurrentMultiplier.size() && !push; ++i){
      if (first != CurrentMultiplier[i]){
	push = true;
      }
    }
    if (!push && (first == 1 || first == -1)){
      push = true;
    }  

    // next test if number of 0's is 1 less than the size                                                   
    // if so, then don't push                                                                               
    int numzerosinmult = 0;
    for (int i = 0; i < CurrentMultiplier.size(); ++i){
      if (CurrentMultiplier[i] == 0){
        ++numzerosinmult;
      }
    }
    if (numzerosinmult == CurrentMultiplier.size() - 1){
      push = false;
    }

    // now test if CurrentMultiplier is a multiple of another multiplier                                    
    // meaning that the gcd is not 1                                                                        
    int multgcd = 0; // set to 0 if numzerosinmult is less than curmult.size()                              
    // i.e. if push is currently false, multgcd won't be set                                                
    if (push){
      multgcd = gcd(CurrentMultiplier);
      //      std::cout << "multgcd = " << multgcd << "\n";
      if (multgcd != 1){
        push = false;
      }
    }

    if (push){
      // push the SignPattern that has the first positive entry
      // if the #POS = #NEG - not implemented
      AllSigns.push_back(CSign);
    }
    /*    else{
      std::cout << "Did not push: [";
      for (int l = 0; l < CSign.IntVec.size(); ++l){
	std::cout << CSign.IntVec[l] << (l != CSign.IntVec.size() - 1 ? ", " : "]\n");
      }
    }
    */
    CurrentMultiplier = NextMultiplier(CurrentMultiplier, MaxInEachDimension);
  }
  
  //std::cout << "Inside ComputeBestSigns(std::vector<SignPattern OriginalBasis)\n";
  for (int i = 0; i < AllSigns.size(); ++i){
    //   std::cout << "[";
    for (int j = 0; j < AllSigns[i].IntVec.size(); ++j){
      //   std::cout << AllSigns[i].IntVec[j] << (j != AllSigns[i].IntVec.size() - 1 ? ", " : "]\n");
    }
  }

  std::vector<SignPattern> TheBestSigns = BestSigns(AllSigns, OriginalBasis.size());
  
  
  //  std::cout << "Inside ComputeBestSigns(std::vector<SignPattern OriginalBasis)\n";
  //  std::cout << "TheBestSigns\n";
  //for (int i = 0; i < TheBestSigns.size(); ++i){
    //  std::cout << "[";
    // for (int j = 0; j < TheBestSigns[i].IntVec.size(); ++j){
    //  std::cout << TheBestSigns[i].IntVec[j] << (j != TheBestSigns[i].IntVec.size() - 1 ? ", " : "]\n");
  // }
  // }



  std::vector<SignPattern> TheBest;

  for (int i = 0; i < TheBestSigns.size(); ++i){
    double n1 = Norm(TheBestSigns[i]);
    double n2 = Norm(TheBestSigns[++i]);

    if (n1 == n2){
      // norms are the same ... test if one is a negative of the other                                      
      int neg1 = -1;
      SignPattern Neg2 = neg1*TheBestSigns[i];
      SignPattern Pos1 = TheBestSigns[i-1];
      if (Identifier(Neg2) == Identifier(Pos1)){
        // one is the negative of the next, choose the signpattern                                          
        // with more positive values                                                                        
        int npos1 = Pos(Pos1);
        int npos2 = Pos(TheBestSigns[i]);
        if (npos1 > npos2){
          TheBest.push_back(Pos1);
        }
        else{
          TheBest.push_back(TheBestSigns[i]);
        }
      }
      else{
        TheBest.push_back(TheBestSigns[i-1]);
        TheBest.push_back(TheBestSigns[i]);
      }
    }   
  }
 
  return TheBest;
}

// this is only for l == 2 case
// do the heuristic later for l > 2
/*SortPositiveVertices(std::vector<Vertex> APV){ //, Rational** LC, int l, int n){

  for (i = 0; i < APV.size(); ++i){

  }

  }
*/

double HeuristicRatio(Rational** LC, int l, int n){

  /* std::cout << "Inside HeuristicRatio\n";*/
  std::vector<Vertex> AllPositiveVertices;
  /* std::cout << "LC = \n\n";
  for (int i = 0; i < n; ++i){
    for (int j = 0; j < l + n + 1; ++j){
      std::cout << LC[i][j] << "\t";
    }
    std::cout << "\n";
  }
  */

  AllPositiveVertices = FindAllPositiveVertices(LC, l, n);
  double MinAngle = 4; // this is smaller than anything possible
  double MaxAngle = -1; // this is bigger than anything possible  
  if (l == 2){
    
    // go ahead and create a copy of the current LC and do GaussElim on it
#ifdef debugmem
      std::cout << "\n******\n\nallocating ToGaussianize\n\n******\n";
#endif
    Rational** ToGaussianize = new Rational*[n];
    for (int i = 0; i < n; ++i){
#ifdef debugmem
        std::cout << "\n******\n\nallocating ToGaussianize[" << i << "]\n\n******\n";
#endif
      ToGaussianize[i] = new Rational[n + l + 1];
      for (int j = 0; j < n + l + 1; ++j){
	ToGaussianize[i][j] = LC[i][j];
      }
    }
    GaussianEliminationT(ToGaussianize, n, n + l + 1);
    
    // create a truncated linears matrix that stores the representation of ALL the linears
    
#ifdef debugmem
      std::cout << "\n******\n\nallocating TruncatedLinears\n\n******\n";
#endif
    Rational** TruncatedLinears = new Rational*[n+l]; // was only of size n
    for (int i = 0; i < n; ++i){
#ifdef debugmem
        std::cout << "\n******\n\nallocating TruncatedLinears[" << i << "]\n\n******\n";
#endif
      TruncatedLinears[i] = new Rational[l + 1];
      for (int j = 0; j < l + 1; ++j){
	TruncatedLinears[i][j] = -1*ToGaussianize[i][j+n]; // monomial parameterization needs this -1*
      }
    }
    for (int i = 0; i < l; ++i){
#ifdef debugmem
        std::cout << "\n******\n\nallocating TruncatedLienars[" << i + n << "]\n\n******\n";
#endif
      TruncatedLinears[i+n] = new Rational[l + 1];
      for (int j = 0; j < l + 1; ++j){
	//      std::cout << "i = " << i << ", j = " << j << "\n";
	if (i == j){
	  TruncatedLinears[i+n][j] = 1;
	}
      }
    }
    
    // delete ToGaussianize
    for (int i = 0; i < n; ++i){
      delete[] ToGaussianize[i];
    }
    delete[] ToGaussianize;
    
    // std::cout << "Changes (?) after calling FindAllPositiveVertices ... \n\nLC = \n\n";
    /*for (int i = 0; i < n; ++i){
      for (int j = 0; j < l + n + 1; ++j){
      std::cout << LC[i][j] << "\t";
      }
      std::cout << "\n";
      }
    */
    // std::cout << "After FindAllPositiveVertices -- inside HeuristicRatio\n";
    //  std::vector<double> AllAngles;
    
    
    
    /*  std::cout << "All the positive Vertices found (there are " << AllPositiveVertices.size() << " of them): \n\n";
	
	for (int i = 0; i < AllPositiveVertices.size(); ++i){
	std::cout << "AllPositiveVertices[" << i << "] = [";
	for (int j = 0; j < AllPositiveVertices[i].first.size(); ++j){
	std::cout << AllPositiveVertices[i].first[j] << (j != AllPositiveVertices[i].first.size() - 1 ? ", " : "]\n");
	}
	}
	std::cout << "\n\n";
    */
    // now, sort the positive vertices so that the ones next to one another, and the last and the first
    // are the ones to to check the angle between the supporting linears
    // this is only for l == 2
    
    //    SortPositiveVertices(AllPositiveVertices, LC, l, n);
    
    
    
    //  std::cout << "Entering i loop\n";
    for (int i = 0; i < AllPositiveVertices.size(); ++i){
      // now for each group of 2-ples that form the angle between the hyperplanes,
      // compute the angle between them                                                                                                              
      
      // std::cout << "Entering j loop\n";
      for (int j = 0; j < AllPositiveVertices[i].second.size(); ++j){
	
	//std::cout << "Entering k loop\n";
	for (int k = j + 1; k < AllPositiveVertices[i].second.size(); ++k){
	  
	  //std::cout << "i, j, k = " << i << ", " << j << ", " << k << "\n\n";
	  
	  // now from here AllPositiveVertices[i].second[j]                                                                                                                                       
	  // and AllPositiveVertices[i].second[k]                                                                                                                                                 
	  // are the rows that form the two linears                                                                                                                                               
	  // in the LC array                                                                                                                                                                      
	  int row1 = AllPositiveVertices[i].second[j];
	  int row2 = AllPositiveVertices[i].second[k];
	  
	  std::cout << "row1 = " << row1 << ", row2 = " << row2 << "\n";
	  
	  // take the dot product of the two vectors                                                                                                                                              
	  // and divide by the norm of each                                                                                                                                                       
	  Rational MyDotProd = 0;
	  Rational Norm1 = 0;
	  Rational Norm2 = 0;
	  std::cout << "We want to find the angle between these two vectors:\n\n";
	  std::cout << "[";
	  for (int t = 0; t < l; ++t){
	    std::cout << TruncatedLinears[row1][t] << (t != l -1? ", " : "]\n\n");
	  }
	  
	  std::cout << "[";
	  for (int t = 0; t < l; ++t){
	    std::cout << TruncatedLinears[row2][t] << (t != l -1? ", " : "]\n\n");
	  }
	  
	  //	std::cout << "Entering t loop\n";
	  for (int t = 0; t < l; ++t){
	    
	    //  std::cout << "t = " << t << "\n";
	    // test if row1 or row2 > numlinear, i.e. if row1 or row2 should be one of the
	    // the linear that represents the coordinate axes
	    
	    Rational entry_row1(0);
	    Rational entry_row2(0);
	    
	    
	    entry_row1 = TruncatedLinears[row1][t];
	    entry_row2 = TruncatedLinears[row2][t];
	    //  std::cout << "entry_row1 = " << entry_row1 << ", entry_row2 = " << entry_row2 << "\n";
	    
	    MyDotProd+= (entry_row1*entry_row2);
	    Norm1 += (entry_row1*entry_row1);
	    Norm2 += (entry_row2*entry_row2);
	    
	    
	  } // end for t
	  
	  //	std::cout << "Leaving t loop\n";
	  
	  Rational MultNorms = Norm1*Norm2;
	  
	  
	  std::cout << "MyDotProd = " << MyDotProd << "\n";
	  std::cout << "Norm1 = " << Norm1 << "\n";
	  std::cout << "Norm2 = " << Norm2 << "\n";
	  // now we need to take the square root of Norm1*Norm2                                                                                                                                   
	  // i.e. convert to double as we cannot take square roots of                                                                                                                             
	  // rationals symbolically :(                                                                                                                                                            
	  
	  double multnormasdouble = double(MultNorms.GetNumerator()) / double(MultNorms.GetDenominator());
	  double newdenom = double(pow(multnormasdouble, .5));
	  double dotprodasdouble = double(MyDotProd.GetNumerator()) / double(MyDotProd.GetDenominator());
	  double cosinv = dotprodasdouble / newdenom;
	  std::cout << "multnormasdouble = " << multnormasdouble << "\n"
		    << "newdenom = " << newdenom << "\n"
		    << "dotprodasdouble = " << dotprodasdouble << "\n";
	  std::cout << "cosinv = " << cosinv << "\n";
	  std::cout << "Before acos\n";
	  double angle = acos(cosinv); // guarantees the angle is from 0 to pi as this is what
	  // the acos function returns
	  std::cout << "After acos\n";
	  std::cout << "angle = " << angle << "\n";
	  if (angle < MinAngle){
	    MinAngle = angle;
	  }
	  if (angle > MaxAngle){
	    MaxAngle = angle;
	  }
	  
	} 
	//     std::cout << "Leaving k loop\n";
	// end for k
      } // end for j 
      // std::cout << "Leaving j loop\n";
    } // end for i
    // std::cout << "Leaving i loop\n";
    
  } // end l == 2
  
  std::cout << "MinAngle = " << MinAngle << ", MaxAngle = " << MaxAngle << "\n";
  
  return MinAngle/MaxAngle; // to avoid division by 0!

}



int WriteMapleOutput(std::string filename, int l, int m, int n, Polynomial* Parametrized, int** exp, int** mon_exp, Polynomial* All_Poly, Variable* All_Vars, Rational** L){

  // filename is the filename of the maple file to create for the run
  // and 'filename'"-runplots.maple" will created for the drawing one, assuming, l == 2 (for now)
  
  std::stringstream filenamess;
  filenamess << filename << ".maple";

  std::ofstream fout(filenamess.str().c_str());  // the general run one
  
  std::stringstream runplotsss;
  runplotsss << filename << "-runplots.maple";

  std::ofstream fout2(runplotsss.str().c_str());  // the drawing one
  // the maple file that will do heuristics
  
    
  // get the galeDuality base directory
  // this is provided by the user when installing
  // and running configure
  // assumes the directory is the 'include' directory in the main galeDuality code
  // otherwise, it is where the user decides to install that

    

 /*   char* galePath;
    galePath = getenv("GALEDUALDIR");
    
    if (galePath !=NULL){
        std::cerr << "You did not add the environment variable GALEDUALDIR into your shell (bash, csh, tcsh, etc.).\n  Follow the instructions in the manual.\n";
        return -1;
    }
    
    std::cout << "Before outputing galePathStr\n";
  */
  system("echo ${GALEDUALDIR} > gale.dir");
    std::ifstream findir("gale.dir");
    std::string galedir;
    getline(findir, galedir);
    //    std::cout << "galePath location = " << &galePath << "\n";
//    std::string galePathStr = galePath;
 //   std::cout << galePathStr << "\n";
    std::string galebasedir = "/Users/matthewniemerg/Research/GaleDuality/Newer/Seamless/dev/include";
    std::cout << galebasedir << "\n"
              << galedir << "\n";
    findir.close();
  system("rm gale.dir");
  fout << "Digits := 40:\n"
       << "interface(quiet = true):\n"
       << "VERBOSE := 0:\n"
       << "final_acc := 1e-11:  # desired final tolerance\n"
       << "starting_delta_t := 0.025:  # Starting steplength.\n"
       << "read(\"" << galedir << "/maple_procedures/main.maple\"):\n"
       << "read(\"" << galedir << "/maple_procedures/drawing.maple\"):\n"
       << "read(\"" << galedir << "/maple_procedures/unbounded.maple\"):\n";

  fout << "# Data Structure\n"
       << "#\n"
       << "# P    is the list of m linear factors in the master function\n"
       << "#\n"
       << "# Exp  is the 2 by m matrix of integer exponents" 
       << " in the master functions\n"
       << "#\n"
       << "# A    is the matrix that represents the exponents of the monomials"
       << "#\n"
       << "# Original_Poly   is the original polynomial system"
       << "#\n"
       << "# Co  is the pair of coefficients\n"
       << "#\n"
       << "# C is the coefficient matrix of the linears, used to test unboundedness\n"
       << "\n"
       << "# X here is always [s1,s2,...,s(l+m)]\n"
       << "#\n"
       << "#  The master functions are \n"
       << "#\n"
       << "# \\prod_j  P[j]^E[i,j] = Coeff[i]   for i=1,2,...,l\n"
       << "#\n"
       << "#    These are global variables\n"
       << "#\n"
       << "############################################################\n";
  
  fout << "l := " << l
       << ":\nm:=" << m
       << ":\nn:=" << n
       << ":\n";
  fout << "\n\n"
       << "P := [ ";
  for (int i = 0; i < l+m+n;++i){
    fout << Parametrized[i];
    if (i != l+m+n-1){
      fout << ", ";
    }
  }
  fout << "]:";
  fout << "\n";
  fout << "Exp := [[";
  for (int i = 0; i < l+m; ++i){
    for (int j = 0; j < l+m+n;++j){
      fout << exp[i][j];
      if (j != l+m+n-1){
        fout << ", ";
      }
    }
    if (i != l+m-1){
      fout << "],\n[";
    }
  }
  fout << "]]:";
  fout << "\n"
       << "A := [[";
  for (int i = 0; i < n; ++i){
    for (int j = 0; j < l+m+n; ++j){
      fout << mon_exp[i][j]
           << (j == l+m+n - 1 ? "" : ", ");
    }
    fout << " ]" << (i == n - 1 ? "": ", [");
    
  }
  fout << "]:\n";
  
  fout << "Original_Poly := [";
  for (int i = 0; i < n; ++i){
    fout << All_Poly[i];
    fout << (i != n-1 ? "," : "]:") << "\n";
  }
  fout << "\n";
  fout << "Original_Vars := [";
  for (int i = 0; i < m+n; ++i){
    fout << All_Vars[i];
    fout << (i != n-1 ? "," :"]:");
  }
  fout << "\n";
  fout << "Co := [";
  for (int i = 0; i < l+m;++i){
    fout << "1";
    if (i != l+m-1){
      fout << ", ";
    }
  }
  fout << "]:\n";
  
  fout << "X := [";
  for (int i=0; i < l+m;++i){
    fout << "s" << i;
    if (i != l+m-1){
      fout <<  ", ";
    }
  }
  fout << "]:\n";

  
  fout << "C := [[";
  for (int i = 0; i < n; ++i){
    for (int j = 0; j < l +m; ++j){
      fout << L[i][j] << (j != l + m - 1 ? ", " : "]");
    }
    fout << (i != n - 1 ? ", [" : "]:\n");
  }
 // fout << "print(Matrix(C));\n";
  fout << "print(\"before project\");\n";
  fout << "# just project anyway . . . until rewrite can be done.\n";
  fout << "# but should be easy to implement for l = 2, with maple's simplex method.  if it's unbounded, the maximize function returns NULL\n";
  fout << "project := false; #true; #IsBounded(Transpose(Matrix(C)));\n";
  fout << "print(\"done figuring out to project\");\n";
  fout << "# note that this only works when we choose the canonical positive chamber\n"
       << "# both the constant and the choice of the hyperplane will be different\n"
       << "# although, this generalizes when we do an appropriate mirror transformations across coordinate axes to move the\n"
       << "# chamber of interest to the positive orthant\n"
       << "if project then\n"
       << "   MyPositiveVertices := FindPositiveVertices(P, X);\n"
       << "   MyC := -1:\n"
       << "   #print(MyPositiveVertices);\n"
       << "   for ll from 1 to nops(MyPositiveVertices) do\n"
       << "      #print(MyPositiveVertices[ll]);\n"
       << "      curvertex := subs(MyPositiveVertices[ll], X);\n"
       << "      #print(curvertex);\n"
       << "      cursum := add(curvertex[i], i = 1..nops(curvertex)):\n"
       << "      if cursum > MyC then\n"
       << "         MyC := cursum:\n"
       << "      end if:\n"
       << "   end do:\n"
       << "   # add an extra 1, because MyC could be 0, and shifting this bounding linear a little 'up' won't change that it's still all positive in Delta\n"
       << "   P := [op(P), 1 - ";
  for (int i = 0; i < l + m; ++i){
    fout << "s" << i << (i != l + m - 1 ? " - " : "]:\n");
  }

  fout << "   for ll from 1 to nops(Exp) do\n";
  fout << "      sumc := 0:\n";
  fout << "      for zz from 1 to nops(Exp[ll]) do\n"
       << "         sumc := sumc + Exp[ll][zz];\n";
  //  for (int i = 0; i < l + m + n; ++i){
  //   fout << "Exp[" << i + 1 << "]" 
  //	 << (i != l + m + n - 1 ? " + " : ":\n");
  // }
  fout << "      end do:\n";
  fout << "      Exp[ll] := [op(Exp[ll]), -sumc]:\n";
  fout << "   end do:\n";
    
  fout << "   # transform the P's -- minus the last one\n"
       << "   #PProjected := [];\n"
       << "   #subvars := [s0=0, s1=0]:\n"
       << "   #for ll from 1 to nops(P) - 1 do \n"
       << "   #   someconst := subs(subvars, P[ll]):\n"
       << "   #   asum := add(X[i], i=1..nops(X)):\n"
       << "   #   PProjected := [PProjected[], P[ll] - someconst*asum]:\n"
       << "   #end do:"
       << "   #PProjected := [PProjected[], P[nops(P)]]:\n"
       << "   #P := PProjected;\n";
    
    
  fout << "end if:\n";
  fout << "print(P);\n"
       << "print(Exp);\n";
  fout << "st := time[real]():";
  fout << "RES := main(project):\n";
  fout << "et :=  time[real]():\n";



  fout << "printf(\"Run complete.\");\n";
  fout << "printf(\"Runtime is : \");\n";
  fout << "printf(\"%a \\n \",et-st);\n";
  fout << "Fac := [[";
  for (int i = 0; i < l+m;++i){
    for (int j = 0; j < l+m+n; ++j){
      fout << "1" << (j != l+m+n-1 ? "," : "]");
    }
    fout << (i != l+m-1 ? ",[" : "]:\n");
  }
  


  fout << "quit;";
  
  // Write the drawing portion of the file ... (this WILL WORK!)
  //  fout2 << "read(\"./maple_procedures/drawing.maple\")"; 

  fout2 << "\n\n"
        << "with(plots):\n"
        << "Linears := [ ";
  for (int i = 0; i < l+m+n;++i){
    fout2 << Parametrized[i];
    if (i != l+m+n-1){
      fout2 << ", ";
    }
  }
  fout2 << "]:";
  fout2 << "\n";
  
  fout2 << "X := [";
  for (int i=0; i < l+m;++i){
    fout2 << "s" << i;
    if (i != l+m-1){
      fout2 <<  ", ";
    }
  }
  fout2 << "]:\n";


  fout2 << "Exp := [[";
  for (int i = 0; i < l+m; ++i){
    for (int j = 0; j < l+m+n;++j){
      fout2 << exp[i][j];
      if (j != l+m+n-1){
        fout2 << ", ";
      }
    }
    if (i != l+m-1){
      fout2 << "],\n[";
    }
  }
  fout2 << "]]:";
  fout2 << "\n";


  fout2 << "Fac := [[";
  for (int i = 0; i < l+m;++i){
    for (int j = 0; j < l+m+n; ++j){
      fout2 << "1" << (j != l+m+n-1 ? "," : "]");
    }
    fout2 << (i != l+m-1 ? ",[" : "]:\n");
  } 

  fout2 << "\n";
  fout2 << "pv:=[]:\n"
        << "for i from 1 to nops(Linears) - 1 do\n"
        << "    for j from i+1 to nops(Linears) do\n"
        << "        csolve:=solve({Linears[i]=0,Linears[j]=0},X);\n"
        << "        if nops(csolve) <> 0 then\n"
        << "            keepgoing := true:\n"
        << "            if keepgoing then\n"
        << "                for k from 1 to nops(Linears) do\n"
        << "                    m:=subs(csolve[1],Linears[k]);\n"
        << "                        if m < 0 then\n"
        << "                            keepgoing:=false;\n"
        << "                        end if:\n"
        << "                end do:\n"
        << "                if keepgoing then\n"
        << "                    pv:=[op(pv),csolve[1]];\n"
        << "                end if:\n"
        << "            end if:\n"
        << "         end if:\n"
        << "    end do:\n"
        << "end do:\n"
        << "uppers:=[0,0]:\n"
        << "for i from 1 to nops(pv) do\n"
        << "    current:=subs(pv[i],X):\n"
        << "    if current[1] > uppers[1] then\n"
        << "        uppers[1] := current[1]:\n"
        << "    end if:\n"
        << "    if current[2] > uppers[2] then\n"
        << "        uppers[2] := current[2]:\n"
        << "    end if:\n"
        << "end do:\n"
        << "#plot the linears\n\n"
        << "plotlin:=implicitplot(Linears,s0=-.5..uppers[1]+.5,s1=-.5..uppers[2]+.5,color=black);\n"
        << "#compute f,g,J1,J2\n"
        << "numlinears:=nops(Linears);\n"    
        << "ftemp:=1;\n"
        << "gtemp:=1;\n"
        << "for i from 1 to nops(Linears) do\n"
        << "ftemp:=ftemp*Fac[1][i]*Linears[i]^(Exp[1][i]):\n"
        << "gtemp:=gtemp*Fac[2][i]*Linears[i]^(Exp[2][i]):\n"
        << "end do:\n"
        << "f:=numer(ftemp)-denom(ftemp);\n"
        << "g:=numer(gtemp)-denom(gtemp):\n"
        << "logf:=0;\n"
        << "logg:=0;\n"
        << "for i from 1 to nops(Linears) do\n"
        << "   logf:=logf+Fac[1][i]*Exp[1][i]*log(Linears[i]):\n"
        << "   logg:=logg+Fac[2][i]*Exp[2][i]*log(Linears[i]):\n"
        << "end do:\n"
        << "#logf:=add(Fac[1][i]*Exp[1][i]*log(Linears[i]),i=1..numlinears):\n"
        << "#logg:=add(Fac[2][i]*Exp[2][i]*log(Linears[i]),i=1..numlinears):\n"
        << "J2:=(diff(logf,s0)*diff(logg,s1)-diff(logf,s1)*diff(logg,s0)):\n"
        << "for i from 1 to nops(Linears) do\n"
        << "     J2:=J2*Linears[i]:\n"
        << "end do:\n"
        << "J1 := (diff(J2,s0)*diff(logf,s1)-diff(J2,s1)*diff(logf,s0)):\n"
        << "for i from 1 to nops(Linears) do\n"
        << "     J1:=J1*Linears[i]:\n"
        << "end do:\n"
        << "# plot logf and logg\n"
        << "plotf:=implicitplot(f,s0=-0.5..uppers[1]+.5,s1=-0.5..uppers[2]+.5,color=red,grid=[200,200],rational=true):\n"
        << "plotg:=implicitplot(g,s0=-0.5..uppers[1]+.5,s1=-0.5..uppers[2]+.5,color=blue,grid=[200,200],rational=true):\n"
        << "\n# plot the jacobians \n\n"
        << "plotJ2:=implicitplot(J2,s0=-0.5..uppers[1]+.5,s1=-0.5..uppers[2]+.5,color=green,grid=[200,200],rational=true):\n"
        << "plotJ1:=implicitplot(J1,s0=-0.5..uppers[1]+.5,s1=-0.5..uppers[2]+.5,color=magenta,grid=[200,200],rational=true):\n"
        << "plot1:=plots[display]([plotlin,plotf,plotg]):\n"
        << "plot2:=plots[display]([plotlin,plotf,plotJ2]):\n"
        << "plot3:=plots[display]([plotlin,plotJ2,plotJ1]):\n";
  



    return 0;

  
}
#endif // end defining of GALEFUNCTIONS_H_
