// 
// class Rational represents rational numbers
//

#ifndef RATIONAL_H
#define RATIONAL_H

#include <iostream>
#include <sstream>
using namespace std;

class Rational {
 public:
  // constructors
  Rational();
  Rational(int numer, int denom = 1);
  Rational(const Rational & R);

  // Inspectors

  int GetNumerator() const;
  int GetDenominator() const;

  // Arithmetic facilitators
  Rational Add(const Rational & r) const;
  Rational Subtract(const Rational & r) const;
  Rational Multiply(const Rational & r) const;
  Rational Divide(const Rational & r) const;

  // stream I/O facilitators
  void Extract(istream & sout);
  void Insert(ostream & sout) const;

  // Comparison facilitator
  // Compare(r)  returns 0 if the receiver is equal to r
  //                    -1 if the receiver is less than r
  //                     1 if the receiver is greater than r
  int Compare(const Rational & r) const;
  void SetNumerator(int n);
  void SetDenominator(int d);
  
  
  Rational& operator=(const Rational& r) {
    if (this != &r) {  // make sure not same object      
      this->NumeratorValue = r.GetNumerator();
      this->DenominatorValue = r.GetDenominator();      
      //     std::cout << "In Rational Assignment operator ... " << std::endl;
    }
    return *this;    // Return ref for multiple assignment
  }

 protected:

  // mutators

  void Reduce();

 private:
  // data members
  int NumeratorValue;
  int DenominatorValue;
};

// Rational ADT: auxiliary operator descriptions
Rational operator + (const Rational & r, const Rational & s);
Rational operator * (const Rational & r, const Rational & s);
Rational operator - (const Rational & r, const Rational & s);
Rational operator / (const Rational & r, const Rational & s);

bool operator < (const Rational & r, const Rational & s);
bool operator > (const Rational & r, const Rational & s);
bool operator == (const Rational & r, const Rational & s);
bool operator <= (const Rational & r, const Rational & s);
bool operator >= (const Rational & r, const Rational & s);
bool operator != (const Rational & r, const Rational & s);

void operator += (Rational & r, const Rational & s);
void operator *= (Rational & r, const Rational & s);
void operator -= (Rational & r, const Rational & s);
void operator /= (Rational & r, const Rational & s);

ostream & operator << (ostream & sout, const Rational & r);
istream & operator >> (istream & sin, Rational & r);

int gcd(int a, int b);

#endif
