//
// Implementation of class Rational
//

// #include "stdafx.h"

#include <iostream>
#include "rational.h"
// #include "gcd.h"
#include <cassert>

// default constructor
Rational::Rational() {

  //  cout << "***entering default constructor***"
  //   << endl;
  NumeratorValue = 0;
  DenominatorValue = 1;
  //  Reduce();

  //  cout << "***leaving default constructor***"
  //       << endl << endl;
}


// (numer, denom) constructor
Rational::Rational(int numer, int denom) {
  NumeratorValue = numer;
  DenominatorValue = denom;
  Reduce();
}

// Copy constructor 

Rational::Rational(const Rational & R) {

  //  cout << endl << endl
  //       << "***entering copy constructor***"
  //       << endl;

  NumeratorValue = R.GetNumerator();
  DenominatorValue = R.GetDenominator();
  Reduce();

  //  cout << "***leaving copy constructor***"
  //       << endl;

}

// get the numerator
int Rational::GetNumerator() const {
  return NumeratorValue;
}


// get the denominator
int Rational::GetDenominator() const {
  return DenominatorValue;
}


// set the numerator
void Rational::SetNumerator(int numer) {
  NumeratorValue = numer;
  Reduce();
}


// set the denominator
void Rational::SetDenominator(int denom) {
  // Check for valid denominator
  assert (denom != 0);
  // Assign denominator
  DenominatorValue = denom;
  Reduce();
}

// Reduce to lowest terms and make sure denominator is positve
void Rational::Reduce() {
  // std::cout << "Numerator = " << GetNumerator() << ", Denominator = " << GetDenominator()
  //    << std::endl;
  int Divisor = 1;
  if (GetNumerator() < 0 && GetDenominator() > 0){
    // std::cout << "Numerator = " << GetNumerator() 
    //      << ", Denominator = " << GetDenominator()
    //      << std::endl;
    Divisor = gcd(-1*GetNumerator(), GetDenominator());
  } 
  else if (GetNumerator() < 0 && GetDenominator() < 0){
    
    // SetNumerator(-1*GetNumerator());
    NumeratorValue = -1*NumeratorValue;
    // SetDenominator(-1*GetDenominator());
    DenominatorValue = -1*DenominatorValue;
    Divisor = gcd(GetNumerator(), GetDenominator());
  }
  else if (GetNumerator() > 0 && GetDenominator() < 0){
    // SetNumerator(GetNumerator()*-1);
    //SetDenominator(GetDenominator()*-1);
    NumeratorValue = -1*NumeratorValue;
    DenominatorValue = -1*DenominatorValue;
    Divisor = gcd(-1*GetNumerator(), GetDenominator());
  }
  else if (GetNumerator() > 0 && GetDenominator() > 0){
    Divisor = gcd(GetNumerator(), GetDenominator());
  }
  // std::cout << "Divisor = " << Divisor << std::endl;
  NumeratorValue /= Divisor;
  DenominatorValue /= Divisor;
  //  std::cout << "Numerator = " << GetNumerator() << ", Denominator = " << GetDenominator()
  //    << std::endl;
  if (NumeratorValue == 0){
    DenominatorValue = 1;
  }
}

// adding Rationals
Rational Rational::Add(const Rational &r) const {
  int a = GetNumerator();
  int b = GetDenominator();
  int c = r.GetNumerator();
  int d = r.GetDenominator();
  return Rational(a*d + b*c, b*d);
}


// multiplying Rationals
Rational Rational::Multiply(const Rational &r) const {
  int a = GetNumerator();
  int b = GetDenominator();
  int c = r.GetNumerator();
  int d = r.GetDenominator();
  return Rational(a*c, b*d);
}

// subtracting Rationals
Rational Rational::Subtract (const Rational &r) const {

  int a = GetNumerator();
  int b = GetDenominator();
  int c = r.GetNumerator();
  int d = r.GetDenominator();

  return Rational(a*d - b*c, b*d);
}


// dividing Rationals
Rational Rational::Divide(const Rational &r) const {
  
  // std::cout << "Entering Divide" << std::endl;
  int a = GetNumerator();
  int b = GetDenominator();
  int c = r.GetNumerator();
  int d = r.GetDenominator();
  // std::cout << "a*d = " << a*d << ", b*c = " << b*c << std::endl;
  //std::cout << "Leaving Divide" << std::endl;
  return Rational(a*d, b*c);
}

// comparing Rationals
int Rational::Compare(const Rational & r) const {

  Rational s(GetNumerator(), GetDenominator());

  if (r == s)
    return 0;
  else if (r > s)
    return -1;
  else 
    return 1; 
}  

// inserting a Rational
void Rational::Insert(ostream &sout) const {
  
  //std::cout << "Entering Rational::Insert(ostream & sout) const" << std::endl;
  

  if (GetDenominator() != 1){
    if (GetNumerator()){
      sout << GetNumerator() << '/' << GetDenominator();
    }
    else{
      sout << GetNumerator();
    }
    // std::cout << "Leaving Rational::Insert(ostream & sout) const" << std::endl;
    return;
  }
  else {
    sout << GetNumerator();
    // std::cout << "Leaving Rational::Insert(ostream & sout) const" << std::endl;
    return;
  }
}

// extracting a Rational
void Rational::Extract(istream &sin) {
  int numer;
  int denom;
  char slash;
  // check to see if only a single acceptable value is in the input stream
  // i.e. a whole number
  // just test if a slash exists in sin or not!
  std::string the_string;
  sin >> the_string;
  //  std::cout << "the input string is : " << the_string << std::endl;
  // sin.clear();
  // sin.str(the_string);
  stringstream s;
  s << the_string;



  if (the_string.find("/") != string::npos){
  
    // sin >> numer >> slash >> denom;
    
    //  s << the_string;
    s >> numer >> slash >> denom;
    SetNumerator(numer);
    SetDenominator(denom);
    Reduce();
  }
  else{
    s >> numer;
    SetNumerator(numer);
    SetDenominator(1);;
  }
  return;
}


// *********** Auxiliary operators

// **** Arithmetic
// adding Rationals
Rational operator + (const Rational & r, const Rational & s) {
  return r.Add(s);
}

// multiplying Rationals
Rational operator * (const Rational & r, const Rational & s) {
  return r.Multiply(s);
}

// subtracting Rationals
Rational operator - (const Rational & r, const Rational & s) {
  
  
  return r.Subtract(s); // remove this line
}

// multiplying Rationals
Rational operator / (const Rational & r, const Rational & s) {
  
  // ** Complete this operator
  return r.Divide(s); // remove this line
}

// **** Comparison operators
bool operator < (const Rational & r, const Rational & s) {
  
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();
  
  return a * d < b * c;
}

bool operator > (const Rational & r, const Rational & s) {
  
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();
  // std::cout << "a = " << a << ", b = " << b << ", c = " << c << ", d = " <<  d << "\n\n";
  // std::cout << "\n\nr = " << r << ",s = " << s << "\n\n";
  if (a == 0 && c != 0){
    
    return a < 0;
  }
  else if (a != 0 && c == 0){
    
    return a >0;
  }
  else if (a == 0 && c == 0){
    return false;
  }
  
  return a * d > b * c; 
}


bool operator == (const Rational & r, const Rational & s) {
 
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();

  return a * d == b * c; 

}
bool operator <= (const Rational & r, const Rational & s) {
  
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();


  return a * d <= b * c;
}

bool operator >= (const Rational & r, const Rational & s) {
 
  
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();

  
  return a * d >= b * c;
  
}

bool operator != (const Rational & r, const Rational & s) {
  
  int a = r.GetNumerator();
  int b = r.GetDenominator();
  int c = s.GetNumerator();
  int d = s.GetDenominator();
  
  
  return a * d != b * c;

}

// compound assignments

void operator += (Rational & r, const Rational & s) {
 
  
  r = r.Add(s);
  return;
}

void operator *= (Rational & r, const Rational & s) {
  
  r = r.Multiply(s);
  return;
  
}

void operator -= (Rational & r, const Rational & s) {
  
  r = r.Subtract(s);
  return;
}

void operator /= (Rational & r, const Rational & s) {
  
  r = r.Divide(s);
  return;
}

// inserting a Rational
ostream& operator << (ostream &sout, const Rational &r) {
  r.Insert(sout);
  return sout;
}


// extracting a Rational
istream & operator >> (istream & sin, Rational & r) {
  r.Extract(sin);
  return sin;
}


int gcd(int a, int b){

  if (a < b)
    return gcd(b,a);
  if (b == 0)
    return a;
  else
    return gcd(b, a % b);
}
