
#include <iostream>
#include <string>
#include <vector>
#include "rational.h"
#include <fstream>
#include "Variable.h"
#include "Monomial.h"
#include "Polynomial.h"
#include "functions.h"
#include "galefunctions.h"
#include <map>
#include <iomanip>

int main(){
  
  
  std::string galeinput;
  std::cout << "Enter the string of the galeinput: ";
  std::cin >> galeinput;

  Variable* All_Variable_Ptr = NULL;
  Rational** LC = NULL;
  int** exponents = NULL;
  int numparam;
  int numlinear;
  int result = ParseInputGale(galeinput, All_Variable_Ptr, LC, exponents, 
			      numparam, numlinear);
  std::vector<std::string> TheMasterFunctions = GaleSystemToString(All_Variable_Ptr, LC, exponents, numparam, numlinear);
  for (int i = 0; i < TheMasterFunctions.size(); ++i){

    std::cout << TheMasterFunctions[i] << "\n";
      
  }
  WriteGaleSystemToBertini(galeinput, All_Variable_Ptr, LC, exponents, numparam, numlinear);
  std::string solutionfilename;
  std::cout << "Enter the filename of the solution to test: ";
  std::cin >> solutionfilename;
  std::vector<std::vector<std::pair<double, double> > > AllSolutions = ReadUnique(solutionfilename, numparam);

  // make sure everything was read in correctly

  for (int i = 0; i < AllSolutions.size(); ++i){
    std::cout << "x" << i << " = (";
    for (int j = 0; j < AllSolutions[i].size(); ++j){
      std::cout << AllSolutions[i][j].first << " + " << AllSolutions[i][j].second << "*I"
		<< (j != AllSolutions[i].size() - 1 ? ", " : ")\n");
    }
  }
  
  std::vector <std::vector<std::pair<double,double> > > LinearsEvaluated = EvaluateAll(AllSolutions, 
										      LC,
										      numlinear);
  

  std::cout << "numlinear = " << numlinear << "\n";
  std::cout << "Evaluation of the linears: \n\n";
  for (int i = 0; i < LinearsEvaluated.size(); ++i){
    std::cout << "x_" << i+1 << "* \n";
    for (int j = 0; j < LinearsEvaluated[i].size(); ++j){
      std::cout << setprecision(16) 
		<< "L" << i+1 << " = " << LinearsEvaluated[i][j].first 
		<< " + " <<  LinearsEvaluated[i][j].second << "*I" << "\n";
    }
  }
  
  // read in the exponents from a pre-defined file -- for use with this testing program!
  // testing the pentagon_20 example

  std::ifstream finexp("Exponents");
  std::string theline;
  int** ExponentsOrig = new int*[3];
  for (int i = 0; i < 3; ++i){
    ExponentsOrig[i] = new int[5];
    
    getline(finexp, theline);
    std::stringstream myexpss;
    myexpss << theline;
    for (int j = 0; j < 5; ++j){
      myexpss >> ExponentsOrig[i][j];
    }
  }

  
  std::vector<std::vector<std::pair<double,double> > > SolvedLinears = SolveLinears(ExponentsOrig,
										    LinearsEvaluated,5,2);

  
  std::cout << "NonExponentiated Solutions! \n\n";
  
  for (int i = 0; i < SolvedLinears.size(); ++i){
    std::cout << "(";
    for (int j = 0; j < SolvedLinears[i].size(); ++j){
      std::cout << setprecision(16) 
		<< SolvedLinears[i][j].first 
	//<< " + " << SolvedLinears[i][j].second
	//<< "*I" 
		<< (j != SolvedLinears[i].size() - 1 ? ", " : ")\n");
    }
  }
  
  std::cout << "\n\n******************\n\n"
	    << "*** Exponentiated Solutions!! *** \n\n\n";

  for (int i = 0; i < SolvedLinears.size(); ++i){
    std::cout << "(";
    std::vector<std::pair<double, double> > ExponentiatedSolutions = ExponentiateVector(SolvedLinears[i]);
    for (int j = 0; j < ExponentiatedSolutions.size(); ++j){
      std::cout << setprecision(16) 
		<< ExponentiatedSolutions[j].first  
		<< " + " << ExponentiatedSolutions[j].second << "*I"
		<< (j != ExponentiatedSolutions.size() - 1 ? ", " :")\n");  
    }
  }
  /*

  
  for (int i = 0; i < CurNonExponentiated.size(); ++i){
   std::vector<std::pair<double, double> > ExponentiatedSolutions = ExponentiateVector(CurNonExponentiated[i]);
    std::cout << "Solution_" << i+1 << " = (";
    for (int j = 0; j < ExponentiatedSolutions.size(); ++j){
      std::cout << ExponentiatedSolutions[j].first << " + " << ExponentiatedSolutions[j].second << "*I"
		<< (j != ExponentiatedSolutions.size() - 1 ? ", " :")\n");
    }
  }
  */
  
  return 0;
}
